@@ -223,10 +223,23 @@ impl TypedTensor {
     pub fn fill_f64(size: &[usize], fill_value: f64) -> TypedTensor {
         TypedTensor::Typef64(GenTensor::fill(fill_value, size))
     }
-    pub fn from_record(&mut self, row: usize, record: &[f32]) -> Result<(), &'static str>{
+    pub fn from_record_f32(&mut self, row: usize, record: &[f32]) -> Result<(), &'static str>{
         match self {
-            TypedTensor::Typef32(v1) => {v1.from_record(row, record)},
-            TypedTensor::Typef64(v1) => {v1.from_record(row, record)},
+            TypedTensor::Typef32(v1) => {v1.from_record_f32(row, record)},
+            TypedTensor::Typef64(v1) => {
+                let new_record: Vec<f64> = record.iter().map(|x| *x as f64).collect();
+                v1.from_record_f64(row, &new_record)
+            },
+            //_ => {panic!("should have same tensor type!");},
+        }
+    }
+    pub fn from_record_f64(&mut self, row: usize, record: &[f64]) -> Result<(), &'static str>{
+        match self {
+            TypedTensor::Typef64(v1) => {v1.from_record_f64(row, record)},
+            TypedTensor::Typef32(v1) => {
+                let new_record: Vec<f32> = record.iter().map(|x| *x as f32).collect();
+                v1.from_record_f32(row, &new_record)
+            },
             //_ => {panic!("should have same tensor type!");},
         }
     }
@@ -648,6 +661,13 @@ impl TypedTensor {
             //_ => {panic!("should have same tensor type!");},
         }
     }
+    pub fn prod(&self, dim: Option<&[usize]>, keepdim: bool) -> TypedTensor {
+        match &self {
+            TypedTensor::Typef32(v1) => {TypedTensor::Typef32(v1.prod(dim, keepdim))},
+            TypedTensor::Typef64(v1) => {TypedTensor::Typef64(v1.prod(dim, keepdim))},
+            //_ => {panic!("should have same tensor type!");},
+        }
+    }
     pub fn sum(&self, dim: Option<&[usize]>, keepdim: bool) -> TypedTensor {
         match &self {
             TypedTensor::Typef32(v1) => {TypedTensor::Typef32(v1.sum(dim, keepdim))},
@@ -671,7 +691,7 @@ impl TypedTensor {
     }
     pub fn min(&self, dim: Option<&[usize]>, keepdim: bool) -> TypedTensor {
         match &self {
-            TypedTensor::Typef32(v1) => {TypedTensor::Typef32(v1.max(dim, keepdim))},
+            TypedTensor::Typef32(v1) => {TypedTensor::Typef32(v1.min(dim, keepdim))},
             TypedTensor::Typef64(v1) => {TypedTensor::Typef64(v1.min(dim, keepdim))},
             //_ => {panic!("should have same tensor type!");},
         }
@@ -703,7 +723,30 @@ impl TypedTensor {
     // Comparison Ops
     typed_tensor_method!(all_close);
     // arg_sort
+    pub fn arg_sort(&self, dim: usize, descending: bool)
+                    -> TypedTensor {
+        match &self {
+            TypedTensor::Typef32(v1) => {
+                TypedTensor::Typef32(v1.arg_sort(dim, descending))
+            },
+            TypedTensor::Typef64(v1) => {
+                TypedTensor::Typef64(v1.arg_sort(dim, descending))
+            },
+            //_ => {panic!("should have same tensor type!");},
+        }
+    }
     typed_tensor_method!(eq_t);
+    pub fn equal(&self, b: &TypedTensor) -> bool {
+        match (&self, b) {
+            (TypedTensor::Typef32(v1), TypedTensor::Typef32(b1)) => {
+                v1.equal(b1)
+            },
+            (TypedTensor::Typef64(v1), TypedTensor::Typef64(b1)) => {
+                v1.equal(b1)
+            },
+            _ => {panic!("should have same tensor type!");},
+        }
+    }
     typed_tensor_method!(ge);
     typed_tensor_method!(gt);
     typed_tensor_method!(le);
